//notation: js file can only use this kind of comments
//since comments will cause error when use in webview.loadurl,
//comments will be remove by java use regexp
(function() {
    if (window.WebViewJavascriptBridge) {
        return;
    }

    var messagingIframe;
    var sendMessageQueue = [];
    var receiveMessageQueue = [];
    var messageHandlers = {};

    var CUSTOM_PROTOCOL_SCHEME = 'yy';
    var QUEUE_HAS_MESSAGE = '__QUEUE_MESSAGE__/';

    var responseCallbacks = {};
    var uniqueId = 1;

    function _createQueueReadyIframe(doc) {
        messagingIframe = doc.createElement('iframe');
        messagingIframe.style.display = 'none';
        doc.documentElement.appendChild(messagingIframe);
    }

    function isAndroid() {
        var ua = navigator.userAgent.toLowerCase();
        var isA = ua.indexOf("android") > -1;
        if (isA) {
            return true;
        }
        return false;
    }

    function isIphone() {
        var ua = navigator.userAgent.toLowerCase();
        var isIph = ua.indexOf("iphone") > -1;
        if (isIph) {
            return true;
        }
        return false;
    }

    //set default messageHandler
    function init(messageHandler) {
        if (WebViewJavascriptBridge._messageHandler) {
            throw new Error('WebViewJavascriptBridge.init called twice');
        }
        WebViewJavascriptBridge._messageHandler = messageHandler;
        var receivedMessages = receiveMessageQueue;
        receiveMessageQueue = null;
        for (var i = 0; i < receivedMessages.length; i++) {
            _dispatchMessageFromNative(receivedMessages[i]);
        }
    }

    function send(data, responseCallback) {
        _doSend({
            data: data
        }, responseCallback);
    }

    function registerHandler(handlerName, handler) {
        messageHandlers[handlerName] = handler;
    }

    function callHandler(handlerName, data, responseCallback) {
        _doSend({
            handlerName: handlerName,
            data: data
        }, responseCallback);
    }

    //sendMessage add message, 触发native处理 sendMessage
    function _doSend(message, responseCallback) {
        if (responseCallback) {
            var callbackId = 'cb_' + (uniqueId++) + '_' + new Date().getTime();
            responseCallbacks[callbackId] = responseCallback;
            message.callbackId = callbackId;
        }

        sendMessageQueue.push(message);
        messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://' + QUEUE_HAS_MESSAGE;
    }

    // 提供给native调用,该函数作用:获取sendMessageQueue返回给native,由于android不能直接获取返回的内容,所以使用url shouldOverrideUrlLoading 的方式返回内容
    function _fetchQueue() {
        var messageQueueString = JSON.stringify(sendMessageQueue);
        sendMessageQueue = [];
        //add by hq
        if (isIphone()) {
            return messageQueueString;
            //android can't read directly the return data, so we can reload iframe src to communicate with java
        } else if (isAndroid()) {
            messagingIframe.src = CUSTOM_PROTOCOL_SCHEME + '://return/_fetchQueue/' + messageQueueString;
        }
    }

    //提供给native使用,
    function _dispatchMessageFromNative(messageJSON) {
        setTimeout(function() {
            var message = JSON.parse(messageJSON);
            var responseCallback;
            //java call finished, now need to call js callback function
            if (message.responseId) {
                responseCallback = responseCallbacks[message.responseId];
                if (!responseCallback) {
                    return;
                }
                responseCallback(message.responseData);
                //delete responseCallbacks[message.responseId];
            } else {
                //直接发送
                if (message.callbackId) {
                    var callbackResponseId = message.callbackId;
                    responseCallback = function(responseData) {
                        _doSend({
                            responseId: callbackResponseId,
                            responseData: responseData
                        });
                    };
                }

                var handler = WebViewJavascriptBridge._messageHandler;
                if (message.handlerName) {
                    handler = messageHandlers[message.handlerName];
                }
                //查找指定handler
                try {
                    handler(message.data, responseCallback);
                } catch (exception) {
                    if (typeof console != 'undefined') {
                        console.log("WebViewJavascriptBridge: WARNING: javascript handler threw.", message, exception);
                    }
                }
            }
        });
    }

    //提供给native调用,receiveMessageQueue 在会在页面加载完后赋值为null,所以
    function _handleMessageFromNative(messageJSON) {
        if (receiveMessageQueue) {
            receiveMessageQueue.push(messageJSON);
        } else {
            _dispatchMessageFromNative(messageJSON);
        }
    }

    var WebViewJavascriptBridge = window.WebViewJavascriptBridge = {
        init: init,
        send: send,
        registerHandler: registerHandler,
        callHandler: callHandler,
        _fetchQueue: _fetchQueue,
        _handleMessageFromNative: _handleMessageFromNative
    };

;(function(bridge){
    var commonArgs = ['success', 'fail', 'cancel', 'complete', 'trigger'];

    var handlerCaches = {};
    function on(eventName, args){
        var init = initEvent(eventName);
        var cache = handlerCaches[eventName];

        each(commonArgs, function(handlerName){
            var typeHandler = args[handlerName];
            // 同一个监听器不要重复监听
            if(typeHandler && cache[handlerName].indexOf(typeHandler) == -1) {
                cache[handlerName].push(typeHandler);
            }
        });

        if(init){
            bridge.callHandler('on_' + eventName, {}, handlerCaches[eventName]._handler);
        }

        return args;
    }

    function off(eventName, args){
        var allEmpty = true;

        if(args) {
            var cache = handlerCaches[eventName];

            each(commonArgs, function(handlerName){
                var handlerCache = cache[handlerName];
                var handlerNeedOff = args[handlerName];
                var index = handlerCache.indexOf(handlerNeedOff);
                // 从此方法缓存中删除此handler
                if(index !== -1) handlerCache.splice(index, 1);

                if(handlerCache.length > 0)
                    allEmpty = false;
            });
        }
        if(allEmpty) {
            delete handlerCaches[eventName];
            bridge.callHandler('off_' + eventName, {});
        }
    }

    function initEvent(eventName, force){
        if(handlerCaches[eventName] && !force) return false;
        else {
            handlerCaches[eventName] = {
                success: [],
                fail: [],
                cancel: [],
                complete: [],
                trigger: [],
                _handler: generateHandler(eventName)
            };
            return true;
        }
    }

    function generateHandler(eventName){
        return function handler(responseData){
            if(typeof responseData === 'string') responseData = JSON.parse(responseData);
            var cache = handlerCaches[eventName];
            var status = responseData.status;
            if(status === 'success') executeHandler(cache, status, responseData);
            else if(status === 'fail') executeHandler(cache, status, responseData);
            else if(status === 'cancel') executeHandler(cache, status, responseData);
            executeHandler(cache, 'complete', responseData);
        };
    }

    function executeHandler(cache, handlerName, responseData){
        var handlers = cache[handlerName];
        each(handlers, function(handler){
            handler(responseData);
        });
    }

    function generateMethod (jsName, nativeName){
        bridge[jsName] = function(args){
            args = dealArgs(args || {});
            bridge.callHandler(nativeName, args.data, function(responseData){
                if(typeof responseData === 'string') responseData = JSON.parse(responseData);
                var status = responseData.status;
                if(status === 'success') args.success && args.success(responseData);
                else if(status === 'fail') args.fail && args.fail(responseData);
                else if(status === 'cancel') args.cancel && args.cancel(responseData);
                args.complete && args.complete(responseData);
            });
        };
    }
    function dealArgs(args){
        var data = {};
        each(args, function(arg, key){
            if(commonArgs.indexOf(key) == -1){
                data[key] = args[key];
                delete args[key];
            }
        });
        args.data = data;
        return args;
    }

    function generateEvent (jsEventName, nativeEventName){
        bridge['on' + getAdaptName(jsEventName)] = function(args){
            return on(nativeEventName, args);
        };
        bridge['off' + getAdaptName(jsEventName)] = function(args){
            return off(nativeEventName, args);
        };
    }
    function getAdaptName(name){
        return name.charAt(0).toUpperCase() + name.substr(1);
    }

    function each(arr, callback){
        if(Object.prototype.toString.call(arr) === '[object Array]') {
            for (var i = 0, len = arr.length; i < len; i++) if (callback(arr[i], i, arr) === false) break;
        }
        else {
            for (var key in arr) if (arr.hasOwnProperty(key)) if (callback(arr[key], key, arr) === false) break;
        }
    }

    bridge.addMethods = function(bridgeMethods){
        if(typeof bridgeMethods === 'string') bridgeMethods = [bridgeMethods];
        each(bridgeMethods, function(methodName){
            var names = getJsAndNativeNames(methodName);
            generateMethod(names.js, names.native);
        });
    };

    bridge.addEvents = function(bridgeEvents){
        if(typeof bridgeEvents === 'string') bridgeEvents = [bridgeEvents];
        each(bridgeEvents, function(eventName){
            var names = getJsAndNativeNames(eventName);
            generateEvent(names.js, names.native);
        });
    };

    function getJsAndNativeNames(methodName){
        var nativeMethodName = methodName;
        if(Object.prototype.toString.call(methodName) === '[object Array]'){
            nativeMethodName = methodName[1];
            methodName = methodName[0];
        }
        return {
            native: nativeMethodName,
            js: methodName
        };
    }

})(window.WebViewJavascriptBridge);

    var doc = document;
    _createQueueReadyIframe(doc);
    var readyEvent = doc.createEvent('Events');
    readyEvent.initEvent('WebViewJavascriptBridgeReady');
    readyEvent.bridge = WebViewJavascriptBridge;
    doc.dispatchEvent(readyEvent);
})();